#ifndef __MESSAGE_H_
#define __MESSAGE_H_

#include <string>
#include <iostream>
#include <iterator>
#include <utility>
#include <map>
#include "giventypes.h"


using namespace std;

class ResultSet;

typedef enum _MessageType {
  Success,
  CUInit,
  CUUpdateDBIn,
  CUTestEn,
  CUSetStatus,
  CUResetSw,
  CUQueryDBIn,
  CUUpdateDBOut,
  CUBillCall,
  CUContentsDBOut,
  CLLogin,
  CLFindCustomer,
  CLAddCustomer,
  CLModifyCustomer,
  CLGetLines,
  CLAddLine,
  CLRemoveLine,
  CLChangeCOS,
  CLChangeLDPlan,
  CLQueryCUDB,
  CLQueryOAMDB,
  CLGetDiscounts,
  CLAddDiscount,
  CLRemoveDiscount,
  CLGetLDPlans,
  CLGetLDPlan,
  CLAddLDPlan,
  CLRemoveLDPlan,
  CLAddDiscountToLDPlan,
  CLRemoveDiscountFromLDPlan,
  CLGetBaseRates,
  CLModifyBaseRate,
  CLHardwareTest,
  CLSuspendService,
  CLResumeService,
  CLPayBill,
  CLGetBill,
  CLGetBillDetails,

  CLAddUser,
  CLRemoveUser,
  CLGetGroupMembers,
  CLGetGroupList,
  
  CLDisableCard,
  CLEnableCard,
  CLResetCard,
  //CU messages have a CU prefix
  //Client messages have a CL prefix

  // error messages
  InvalidType = -1000,
  CLLoginFailure,
  CLDBActionFailure,
  CLInsuffAccess,
  CLParameterFailure,
  CLInvalidDBEntry,
  CUInvalidDBEntry,
  CUInvalidTest,
  CLDiscountConflict
}MessageType;


typedef enum _Parameter{
  //CU parameters have a CU prefix
  //client parameters have a CL prefix
  CLCustomerSalutation,
  CLCustomerFirstName,
  CLCustomerLastName,
  CLCustomerAddress1,
  CLCustomerAddress2,
  CLCustomerCity,
  CLCustomerProvince,
  CLCustomerPostalCode,
  CLCustomerDOB,
  CLCustomerSecurityCode,
  
  CLCustomerId,
  CLResultSet,

  CLLoginUserName,
  CLLoginPassword,
  CLGroupId,
  CLGroupName,
  CLAccessMask,
  
  CLEx,
  CLShelf,
  CLSlot,
  CLDN,
  CLCOS,
  CLSTAT,

  CLHardwareStatus,

  CLDiscountName,
  CLDiscountStart,
  CLDiscountEnd,
  CLDiscountRate,
  CLDiscountDayMask,
  CLDiscountId,
  CLLDPlanName,
  CLLDPlanId,
  CLLocalRate,
  CLLDBaseRate,
  CLPayAmount,
  CLBillId,
  CLBillDate,
  CLBillAmount,
  CLBillPastDueFlag,
  CLCallStart,
  CLCallDuration,
  CLCalleeEX,
  CLCalleeDN,
  CLCallDay,
  CLCustomerLanguage,
  CUnType,
  CUEx,
  CUShelf,
  CUSlot,
  CUSTAT,
  CUDN,
  CUCOS,
  CUChangeSTAT,
  CUCallerEx,
  CUCalleeEx,
  CUCallerShelf,
  CUCallerSlot,
  CUDialedPhoneNumber,
  CUTOC,
  CUDuration

  
} Parameter;

typedef map < Parameter, void*, less<Parameter> > MessageMap;

/*
 * Internal OAM messaging class. The Message class simply defines the
 * interface.
 */
class Message{
 protected:
  MessageMap msgMap;
  MessageType type;

  //string equivalents for types;
  static map < MessageType, const char*, less<MessageType> > msgTypeToStrMap;
  static MessageMap paramTypeToStrMap;
  
 public:
  Message() { type = InvalidType; }
  virtual ~Message(){}

  MessageType getType() {return type;}

  void setType(MessageType tp) { type = tp;} 


  /*
   *Returns the values associated with the key
   */
  void* getParameter(Parameter key);
  /*
   *Associates a value with the key
   */
  void setParameter(Parameter key, char* value);
  void setParameter(Parameter key, int value);
  void setParameter(Parameter key, long int value);
  void setParameter(Parameter key, float value);
  void setParameter(Parameter key, char value);
  void setParameter(Parameter key, unsigned char value);
  void setParameter(Parameter key, TimeOfCall value);
  void setParameter(Parameter key, ResultSet* rs);
  void Message::printMessage();
  /*
   *Returns a buffer with the message in the format of the target
   *Out: 
   *	void** buf - this is the buffer that will contain the output
   *		     msg. It will be allocated for you, but it's up
   *		     to you to deallocate.
   *    int* size  - this will contain the size of buf after completion
   *Return:
   *	int - status of the operation
   */
  virtual int toNativeType(void** buf, int* size)=0;
  /*
   *Converts the message from the native format to OAM message format
   *In:
   *	void* buf - buffer containing the message.
   *Return:
   *	int - status of the operation 
   */
  virtual int fromNativeType(void* buf)=0;

 private:
  void* allocateAndCopy(void* source, int length);
};

#endif
