#include "ClientMessage.h"
#include "ReturnTypes.h"
#include "ResultSet.h"

#include "libxml/xmlmemory.h"
#include "libxml/parser.h"

#include "stdlib.h"
#include "stdio.h"

char* itos(int intToConvert)
{
  char* buf = (char*)malloc(10*sizeof(char)); //10 digits for a max int
  sprintf(buf,"%d",intToConvert);
  return buf;
}


int ClientMsg::fromNativeType(void* buf)
{
  xmlDocPtr doc = xmlParseMemory((char*) buf,strlen((char*) buf));

  //check for error here if have time
  //make sure that the correct type of xml

  //iterate over the param nodes
  xmlNodePtr msgNode = xmlDocGetRootElement(doc);
  Message::setType((MessageType)atoi((char*)xmlGetProp(msgNode,
					  XML_CAST CL_MSG_ROOT_ATTR_NAME)));

  xmlNodePtr paramNode = msgNode->xmlChildrenNode;
  
  Parameter key; 

  while(paramNode!=NULL){
    key = (Parameter)atoi((char*)xmlGetProp(paramNode,
				     XML_CAST CL_MSG_PARM_ATTR_NAME));
    
    if(key!=CLResultSet){
      setParameter(key,(char*)xmlNodeGetContent(paramNode));
    }
    else{
      xmlNodePtr cur = paramNode->xmlChildrenNode;
      xmlNodePtr schema = NULL;
      xmlNodePtr rs = NULL;
      //check if schema node
      if(xmlStrcmp(cur->name,
		   XML_CAST CL_MSG_RS_SCHEMA_NODE_NAME)==0){
	schema = cur;
	//and the result set is the next one
	rs = cur->next;
      }
      //otherwise it is the result set node
      else{
	rs = cur;
	//and schema is the next
	schema = cur->next;
      }
      //create the our internal result set from the 
      ResultSet* resultSet = new ResultSet(schema,rs);
      setParameter(key,resultSet);
    }
    
    paramNode = paramNode->next;
  }
  return Success; 
}

int ClientMsg::toNativeType(void**buf, int* size)
{
  if(type==InvalidType){
    return ERR_TYPE_NOT_SET;
  }

  xmlDocPtr doc = xmlNewDoc(XML_CAST"1.0"); //create a new xml documet
  //doc->encoding = xmlStrdup(XML_CAST"iso-8859-1");
  xmlNodePtr root = xmlNewDocNode(doc,
				  NULL,
				  XML_CAST CL_MSG_ROOT_NODE_NAME,
				  NULL);

  xmlDocSetRootElement(doc,root);
  //message type is an attribute on the root node
  xmlAttrPtr newattr = xmlNewProp(root, 
				  XML_CAST CL_MSG_ROOT_ATTR_NAME,
				  XML_CAST itos(type));
  
  //iterate over all the hash map entries and insert them into the xml
  xmlNodePtr paramNode = NULL;

  MessageMap::iterator start = msgMap.begin();

  Parameter paramType;
  const char* paramName;  
  

  //iterate over the hashmap creating the xml as we go along
  while(start!=msgMap.end()){
    paramType = start->first;
    paramName = itos(paramType);
    

    //create a new node
    paramNode = xmlNewTextChild(root,
				NULL,
				XML_CAST CL_MSG_PARM_NODE_NAME, 
				NULL);
    //set the type of the param as an attribute on the node
    newattr = xmlNewProp(paramNode,
			 XML_CAST CL_MSG_PARM_ATTR_NAME,
			 XML_CAST paramName);
    //insert the node into the tree
    if(paramType!=CLResultSet){
      xmlNodeSetContent(paramNode, XML_CAST start->second);
    }
    else {
      xmlNodePtr schema = xmlNewTextChild(paramNode,
					  NULL,
					  XML_CAST CL_MSG_RS_SCHEMA_NODE_NAME,
					  NULL);
      xmlNodePtr rs = xmlNewTextChild(paramNode,
				      NULL,
				      XML_CAST CL_MSG_RS_RS_NODE_NAME,
				      NULL);

      ResultSet* res = (ResultSet*)start->second;

      res->getAsXml(schema,rs);
    }
    start++;
  }


  xmlDocDumpMemory(doc,(xmlChar**)buf,size);
  
  return Success;
}

